@@ -80,8 +80,9 @@ typedef enum {
     EXOSIP_OPT_DONT_SEND_101 = 6,
     EXOSIP_OPT_USE_RPORT = 7,
     EXOSIP_OPT_SET_IPV4_FOR_GATEWAY = 8,
-	EXOSIP_OPT_ADD_DNS_CACHE = 9,
-	EXOSIP_OPT_EVENT_PACKAGE = 10
+    EXOSIP_OPT_ADD_DNS_CACHE = 9,
+    EXOSIP_OPT_EVENT_PACKAGE = 10,
+    EXOSIP_OPT_SET_IPV6_FOR_GATEWAY = 11
 } eXosip_option;
 
 struct eXosip_dns_cache {
@@ -165,6 +166,12 @@ int eXosip_set_option(eXosip_option opt, const void *value);
  */
   void eXosip_set_user_agent(const char *user_agent);
 
+ /**
+  * Get the eXosip version as a sring
+  *
+  */
+  const char * eXosip_get_version(void);
+
 /**
  * Use IPv6 instead of IPv4.
  * 
@@ -187,6 +194,15 @@ int eXosip_set_option(eXosip_option opt, const void *value);
  */
   void eXosip_masquerade_contact(const char *public_address, int port);
 
+/**
+ * This method is used to find out an free IPPROTO_UDP or IPPROTO_TCP port.
+ *
+ * @param free_port          	initial port for search.
+ * @param transport          	IPPROTO_UDP or IPPROTO_TCP protocol.
+ * 
+ */
+  int eXosip_find_free_port(int free_port, int transport);
+
 #ifndef DOXYGEN
 
 /**
