@@ -46,39 +46,21 @@ eXosip_enable_ipv6 (int _ipv6_enable)
 
 #endif
 
+const char *
+eXosip_get_version(void)
+{
+  return EXOSIP_VERSION;
+}
+
 void
 eXosip_masquerade_contact (const char *public_address, int port)
 {
-  if (public_address == NULL || public_address[0] == '\0')
-    {
-      memset (eXosip.net_interfaces[0].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[0].net_firewall_ip));
-      memset (eXosip.net_interfaces[1].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[1].net_firewall_ip));
-      memset (eXosip.net_interfaces[2].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[2].net_firewall_ip));
-      return;
-    }
-
-  snprintf (eXosip.net_interfaces[0].net_firewall_ip,
-            sizeof (eXosip.net_interfaces[0].net_firewall_ip), "%s",
-            public_address);
-  snprintf (eXosip.net_interfaces[1].net_firewall_ip,
-            sizeof (eXosip.net_interfaces[1].net_firewall_ip), "%s",
-            public_address);
-  snprintf (eXosip.net_interfaces[2].net_firewall_ip,
-            sizeof (eXosip.net_interfaces[2].net_firewall_ip), "%s",
-            public_address);
-
-  if (port > 0)
-    {
-      snprintf (eXosip.net_interfaces[0].net_port,
-                sizeof (eXosip.net_interfaces[0].net_port), "%i", port);
-      snprintf (eXosip.net_interfaces[1].net_port,
-                sizeof (eXosip.net_interfaces[1].net_port), "%i", port);
-      snprintf (eXosip.net_interfaces[2].net_port,
-                sizeof (eXosip.net_interfaces[2].net_port), "%i", port);
-    }
+  eXtl_udp.tl_masquerade_contact(public_address, port);
+  eXtl_tcp.tl_masquerade_contact(public_address, port);
+#ifdef HAVE_OPENSSL_SSL_H
+  eXtl_tls.tl_masquerade_contact(public_address, port);
+  eXtl_dtls.tl_masquerade_contact(public_address, port);
+#endif
   return;
 }
 
@@ -156,7 +138,6 @@ eXosip_quit (void)
 #ifdef OSIP_MT
   int i;
 #endif
-  int pos;
 
   if (eXosip.j_stop_ua==-1)
   {
@@ -216,32 +197,6 @@ eXosip_quit (void)
 #endif
 #endif
 
-  if (eXosip.net_interfaces[0].net_socket)
-    {
-      close (eXosip.net_interfaces[0].net_socket);
-      eXosip.net_interfaces[0].net_socket = -1;
-    }
-  if (eXosip.net_interfaces[1].net_socket)
-    {
-      close (eXosip.net_interfaces[1].net_socket);
-      eXosip.net_interfaces[1].net_socket = -1;
-    }
-  if (eXosip.net_interfaces[2].net_socket)
-    {
-      close (eXosip.net_interfaces[2].net_socket);
-      eXosip.net_interfaces[2].net_socket = -1;
-    }
-
-  for (pos = 0; pos < EXOSIP_MAX_SOCKETS; pos++)
-    {
-      if (eXosip.net_interfaces[0].net_socket_tab[pos].socket != 0)
-        close (eXosip.net_interfaces[0].net_socket_tab[pos].socket);
-      if (eXosip.net_interfaces[1].net_socket_tab[pos].socket != 0)
-        close (eXosip.net_interfaces[1].net_socket_tab[pos].socket);
-      if (eXosip.net_interfaces[2].net_socket_tab[pos].socket != 0)
-        close (eXosip.net_interfaces[2].net_socket_tab[pos].socket);
-    }
-
   for (jreg = eXosip.j_reg; jreg != NULL; jreg = eXosip.j_reg)
     {
       REMOVE_ELEMENT (eXosip.j_reg, jreg);
@@ -301,6 +256,13 @@ eXosip_quit (void)
       osip_free (jauthinfo);
     }
 
+  eXtl_udp.tl_free();
+  eXtl_tcp.tl_free();
+#ifdef HAVE_OPENSSL_SSL_H
+  eXtl_dtls.tl_free();
+  eXtl_tls.tl_free();
+#endif
+
   memset (&eXosip, 0, sizeof (eXosip));
   eXosip.j_stop_ua = -1;
   return;
@@ -309,14 +271,23 @@ eXosip_quit (void)
 int
 eXosip_set_socket (int transport, int socket, int port)
 {
-  if (eXosip.net_interfaces[0].net_socket > 0)
-    close (eXosip.net_interfaces[0].net_socket);
-
-  eXosip.net_interfaces[0].net_protocol = transport;
-  eXosip.net_interfaces[0].net_ip_family = AF_INET;
-  eXosip.net_interfaces[0].net_socket = socket;
-  snprintf (eXosip.net_interfaces[0].net_port,
-            sizeof (eXosip.net_interfaces[0].net_port), "%i", port);
+  eXosip.eXtl = NULL;
+  if (transport==IPPROTO_UDP)
+    {
+      eXtl_udp.proto_port = port;
+      eXtl_udp.tl_set_socket(socket);
+      eXosip.eXtl = &eXtl_udp;
+      snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "UDP");
+    }
+  else if (transport==IPPROTO_UDP)
+    {
+      eXtl_tcp.proto_port = port;
+      eXtl_tcp.tl_set_socket(socket); 
+      eXosip.eXtl = &eXtl_tcp;
+      snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "TCP");
+   }
+  else
+    return -1;
 
 #ifdef OSIP_MT
   eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
@@ -341,288 +312,302 @@ setsockopt_ipv6only (int sock)
 }
 #endif /* IPV6_V6ONLY */
 
+#ifndef MINISIZE
+int eXosip_find_free_port(int free_port, int transport)
+{
+  int res1;
+  int res2;
+  struct addrinfo *addrinfo_rtp = NULL;
+  struct addrinfo *curinfo_rtp;
+  struct addrinfo *addrinfo_rtcp = NULL;
+  struct addrinfo *curinfo_rtcp;
+  int sock;
+  int count;
+
+  for (count=0;count<8;count++)
+  {
+	  res1 = eXosip_get_addrinfo (&addrinfo_rtp, "0.0.0.0", free_port + count*2, transport);
+	  if (res1)
+		return -1;
+	  res2 = eXosip_get_addrinfo (&addrinfo_rtcp, "0.0.0.0", free_port + count*2+ 1, transport);
+	  if (res2)
+	  {
+		  eXosip_freeaddrinfo (addrinfo_rtp);
+		  return -1;
+	  }
+
+	  sock=-1;
+	  for (curinfo_rtp = addrinfo_rtp; curinfo_rtp; curinfo_rtp = curinfo_rtp->ai_next)
+		{
+		  if (curinfo_rtp->ai_protocol && curinfo_rtp->ai_protocol != transport)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_INFO3, NULL,
+						   "eXosip: Skipping protocol %d\n", curinfo_rtp->ai_protocol));
+			  continue;
+			}
+
+		  sock = (int) socket (curinfo_rtp->ai_family, curinfo_rtp->ai_socktype,
+							   curinfo_rtp->ai_protocol);
+		  if (sock < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						   "eXosip: Cannot create socket!\n"));
+			  continue;
+			}
+
+		  if (curinfo_rtp->ai_family == AF_INET6)
+			{
+#ifdef IPV6_V6ONLY
+			  if (setsockopt_ipv6only (sock))
+				{
+				  close (sock);
+				  sock = -1;
+				  OSIP_TRACE (osip_trace
+							  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+							   "eXosip: Cannot set socket option!\n"));
+				  continue;
+				}
+#endif /* IPV6_V6ONLY */
+			}
+
+		  res1 = bind (sock, curinfo_rtp->ai_addr, curinfo_rtp->ai_addrlen);
+		  if (res1 < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+						  "eXosip: Cannot bind socket node: 0.0.0.0 family:%d\n",
+						   curinfo_rtp->ai_family));
+			  close (sock);
+			  sock = -1;
+			  continue;
+			}
+		  break;
+	  }
+
+	  eXosip_freeaddrinfo (addrinfo_rtp);
+
+	  if (sock==-1)
+	  {
+		  eXosip_freeaddrinfo (addrinfo_rtcp);
+		  continue;
+	  }
+
+	  close(sock);
+	  sock=-1;
+	  for (curinfo_rtcp = addrinfo_rtcp; curinfo_rtcp; curinfo_rtcp = curinfo_rtcp->ai_next)
+		{
+		  if (curinfo_rtcp->ai_protocol && curinfo_rtcp->ai_protocol != transport)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_INFO3, NULL,
+						   "eXosip: Skipping protocol %d\n", curinfo_rtcp->ai_protocol));
+			  continue;
+			}
+
+		  sock = (int) socket (curinfo_rtcp->ai_family, curinfo_rtcp->ai_socktype,
+							   curinfo_rtcp->ai_protocol);
+		  if (sock < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						   "eXosip: Cannot create socket!\n"));
+			  continue;
+			}
+
+		  if (curinfo_rtcp->ai_family == AF_INET6)
+			{
+#ifdef IPV6_V6ONLY
+			  if (setsockopt_ipv6only (sock))
+				{
+				  close (sock);
+				  sock = -1;
+				  OSIP_TRACE (osip_trace
+							  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+							   "eXosip: Cannot set socket option!\n"));
+				  continue;
+				}
+#endif /* IPV6_V6ONLY */
+			}
+
+		  res1 = bind (sock, curinfo_rtcp->ai_addr, curinfo_rtcp->ai_addrlen);
+		  if (res1 < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+						  "eXosip: Cannot bind socket node: 0.0.0.0 family:%d\n",
+						   curinfo_rtp->ai_family));
+			  close (sock);
+			  sock = -1;
+			  continue;
+			}
+		  break;
+	  }
+
+	  eXosip_freeaddrinfo (addrinfo_rtcp);
+
+	  /* the pair must be free */
+	  if (sock==-1)
+		  continue;
+
+	  close(sock);
+	  sock=-1;
+	  return free_port + count*2;
+  }
+
+  /* just get a free port */
+  res1 = eXosip_get_addrinfo (&addrinfo_rtp, "0.0.0.0", 0, transport);
+  if (res1)
+	return -1;
+
+  sock=-1;
+  for (curinfo_rtp = addrinfo_rtp; curinfo_rtp; curinfo_rtp = curinfo_rtp->ai_next)
+	{
+	  socklen_t len;
+	  struct sockaddr_storage ai_addr;
+
+	  if (curinfo_rtp->ai_protocol && curinfo_rtp->ai_protocol != transport)
+		{
+		  OSIP_TRACE (osip_trace
+					  (__FILE__, __LINE__, OSIP_INFO3, NULL,
+					   "eXosip: Skipping protocol %d\n", curinfo_rtp->ai_protocol));
+		  continue;
+		}
+
+	  sock = (int) socket (curinfo_rtp->ai_family, curinfo_rtp->ai_socktype,
+						   curinfo_rtp->ai_protocol);
+	  if (sock < 0)
+		{
+		  OSIP_TRACE (osip_trace
+					  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+					   "eXosip: Cannot create socket!\n"));
+		  continue;
+		}
+
+	  if (curinfo_rtp->ai_family == AF_INET6)
+		{
+#ifdef IPV6_V6ONLY
+		  if (setsockopt_ipv6only (sock))
+			{
+			  close (sock);
+			  sock = -1;
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						   "eXosip: Cannot set socket option!\n"));
+			  continue;
+			}
+#endif /* IPV6_V6ONLY */
+		}
+
+	  res1 = bind (sock, curinfo_rtp->ai_addr, curinfo_rtp->ai_addrlen);
+	  if (res1 < 0)
+		{
+		  OSIP_TRACE (osip_trace
+					  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+					  "eXosip: Cannot bind socket node: 0.0.0.0 family:%d\n",
+					   curinfo_rtp->ai_family));
+		  close (sock);
+		  sock = -1;
+		  continue;
+		}
+
+      len = sizeof (ai_addr);
+      res1 = getsockname (sock, (struct sockaddr *) &ai_addr, &len);
+      if (res1 != 0)      
+	  {
+		  close (sock);
+		  sock = -1;
+		  continue;
+	  }
+
+	  close(sock);
+	  sock=-1;
+	  eXosip_freeaddrinfo (addrinfo_rtp);
+
+	  if (ipv6_enable == 0)
+		return ntohs (((struct sockaddr_in *) &ai_addr)->sin_port);
+	  else
+		return ntohs (((struct sockaddr_in6 *) &ai_addr)->sin6_port);
+  }
+
+  eXosip_freeaddrinfo (addrinfo_rtp);
+
+  if (sock!=-1)
+  {
+	  close(sock);
+	  sock=-1;
+  }
+
+  return -1;
+}
+#endif
+
 int
 eXosip_listen_addr (int transport, const char *addr, int port, int family,
                     int secure)
 {
-  int res;
-  struct addrinfo *addrinfo = NULL;
-  struct addrinfo *curinfo;
-  const char *node = addr;
-  int sock = -1;
-  struct eXosip_net *net_int;
-  char localip[256];
+  int i=-1;
+  struct eXtl_protocol *eXtl=NULL;
 
-  if (transport == IPPROTO_UDP)
-    net_int = &eXosip.net_interfaces[0];
-  else if (transport == IPPROTO_TCP)
-    net_int = &eXosip.net_interfaces[1];
-  else if (transport == IPPROTO_TCP && secure != 0)
-    net_int = &eXosip.net_interfaces[2];
-  else
+  if (eXosip.eXtl!=NULL)
     {
+      /* already set */
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unknown protocol (use IPPROTO_UDP or IPPROTO_TCP!\n"));
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: already listening somewhere\n"));
       return -1;
     }
 
-#ifndef MINISIZE
-  if (eXosip.http_port)
-    {
-      /* USE TUNNEL CAPABILITY */
-      transport = IPPROTO_TCP;
-    }
+  if (transport==IPPROTO_UDP && secure==0)
+    eXtl = &eXtl_udp;
+  else if (transport==IPPROTO_TCP && secure==0)
+    eXtl = &eXtl_tcp;
+#ifdef HAVE_OPENSSL_SSL_H
+  else if (transport==IPPROTO_UDP)
+    eXtl = &eXtl_dtls;
+  else if (transport==IPPROTO_TCP)
+    eXtl = &eXtl_tls;
 #endif
 
-  if (port < 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: port must be higher than 0!\n"));
-      return -1;
-    }
-
-  net_int->net_ip_family = family;
-#ifndef MINISIZE
-  if (family == AF_INET6)
-    {
-      ipv6_enable = 1;
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "IPv6 is enabled. Pls report bugs\n"));
-    }
-#endif
-
-  eXosip_guess_localip (net_int->net_ip_family, localip, sizeof (localip));
-  if (localip[0] == '\0')
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: No ethernet interface found!\n"));
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: using 127.0.0.1 (debug mode)!\n"));
-      /* we should always fallback on something. The linphone user will surely
-         start linphone BEFORE setting its dial up connection. */
-    }
-
-  if (!node)
-    {
-      node = ipv6_enable ? "::" : "0.0.0.0";
-    }
-
-
-  res = eXosip_get_addrinfo (&addrinfo, node, port, transport);
-  if (res)
+  if (eXtl==NULL)
     return -1;
 
-  for (curinfo = addrinfo; curinfo; curinfo = curinfo->ai_next)
-    {
-      socklen_t len;
+  eXtl->proto_family=family;
+  eXtl->proto_port=port;
+  if (addr!=NULL)
+    snprintf(eXtl->proto_ifs, sizeof(eXtl->proto_ifs), "%s", addr);
 
-      if (curinfo->ai_protocol && curinfo->ai_protocol != transport)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO3, NULL,
-                       "eXosip: Skipping protocol %d\n", curinfo->ai_protocol));
-          continue;
-        }
+  i = eXtl->tl_open();
+  
+  if (i!=0)
+    return -1;
+  
+  eXosip.eXtl = eXtl;
 
-      sock = (int) socket (curinfo->ai_family, curinfo->ai_socktype,
-                           curinfo->ai_protocol);
-      if (sock < 0)
-        {
-#if defined(_WIN32_WCE)
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot create socket!\n"));
-#else
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot create socket %s!\n", strerror (errno)));
-#endif
-          continue;
-        }
-
-#ifndef MINISIZE
-      if (eXosip.http_port)
-        {
-          break;
-        }
-#endif
-
-      if (curinfo->ai_family == AF_INET6)
-        {
-#ifdef IPV6_V6ONLY
-          if (setsockopt_ipv6only (sock))
-            {
-              close (sock);
-              sock = -1;
-#if defined(_WIN32_WCE)
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot set socket option!\n"));
-#else
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot set socket option %s!\n",
-                           strerror (errno)));
-#endif
-              continue;
-            }
-#endif /* IPV6_V6ONLY */
-        }
-
-      res = bind (sock, curinfo->ai_addr, curinfo->ai_addrlen);
-      if (res < 0)
-        {
-#if defined(_WIN32_WCE)
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot bind socket node:%s family:%d\n",
-                       node, curinfo->ai_family));
-#else
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot bind socket node:%s family:%d %s\n",
-                       node, curinfo->ai_family, strerror (errno)));
-#endif
-          close (sock);
-          sock = -1;
-          continue;
-        }
-      len = sizeof (net_int->ai_addr);
-      res = getsockname (sock, (struct sockaddr *) &net_int->ai_addr, &len);
-      if (res != 0)
-        {
-#if defined(_WIN32_WCE)
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot get socket name\n"));
-#else
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot get socket name (%s)\n", strerror (errno)));
-#endif
-          memcpy (&net_int->ai_addr, curinfo->ai_addr, curinfo->ai_addrlen);
-        }
-
-      if (transport != IPPROTO_UDP)
-        {
-          res = listen (sock, SOMAXCONN);
-          if (res < 0)
-            {
-#if defined(_WIN32_WCE)
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot bind socket node:%s family:%d\n",
-                           node, curinfo->ai_family));
-#else
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot bind socket node:%s family:%d %s\n",
-                           node, curinfo->ai_family, strerror (errno)));
-#endif
-              close (sock);
-              sock = -1;
-              continue;
-            }
-        }
-
-      break;
-    }
-
-  eXosip_freeaddrinfo (addrinfo);
-
-  if (sock < 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: Cannot bind on port: %i\n", port));
-      return -1;
-    }
-
-  net_int->net_protocol = IPPROTO_UDP;
-#ifndef MINISIZE
-  if (!eXosip.http_port)
-    net_int->net_protocol = transport;
-#endif
-  net_int->net_socket = sock;
-
-  if (port == 0)
-    {
-      /* get port number from socket */
-      if (ipv6_enable == 0)
-        port = ntohs (((struct sockaddr_in *) &net_int->ai_addr)->sin_port);
-      else
-        port = ntohs (((struct sockaddr_in6 *) &net_int->ai_addr)->sin6_port);
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "eXosip: Binding on port %i!\n", port));
-    }
-
-  snprintf (net_int->net_port, sizeof (net_int->net_port) - 1, "%i", port);
-
-
-
-#ifndef MINISIZE
-  if (eXosip.http_port)
-    {
-      /* only ipv4 */
-      struct sockaddr_in _addr;
-      char http_req[2048];
-      char http_reply[2048];
-      int len;
-
-      _addr.sin_port = (unsigned short) htons (eXosip.http_port);
-      _addr.sin_addr.s_addr = inet_addr (eXosip.http_proxy);
-      _addr.sin_family = PF_INET;
-
-      if (connect
-          (net_int->net_socket, (struct sockaddr *) &_addr, sizeof (_addr)) == -1)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Failed to connect to http server on %s:%i!\n",
-                       eXosip.http_proxy, port));
-          return -1;
-        }
-
-      sprintf (http_req, "GET / HTTP/1.1\r\nUdpHost: %s:%d\r\n\r\n",
-               eXosip.http_outbound_proxy, 5060);
-
-      len = send (net_int->net_socket, http_req, (int) strlen (http_req), 0);
-
-      if (len < 0)
-        return -1;
-
-      osip_usleep (50000);
-
-      if ((len =
-           recv (net_int->net_socket, http_reply, sizeof (http_reply), 0)) > 0)
-        http_reply[len] = '\0';
-      else
-        return -1;
-
-      if (strncmp (http_reply, "HTTP/1.0 200 OK\r\n", 17) == 0
-          || strncmp (http_reply, "HTTP/1.1 200 OK\r\n", 17) == 0)
-        {
-      } else
-        return -1;
-    }
-#endif
+  if (transport==IPPROTO_UDP && secure==0)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "UDP");
+  else if (transport==IPPROTO_TCP && secure==0)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "TCP");
+  else if (transport==IPPROTO_UDP)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "DTLS-UDP");
+  else if (transport==IPPROTO_TCP)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "TLS");
 
 #ifdef OSIP_MT
-  eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
-  if (eXosip.j_thread == NULL)
+  if (eXosip.j_thread==NULL)
     {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+      eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
+      if (eXosip.j_thread == NULL)
+	{
+	  OSIP_TRACE (osip_trace
+		      (__FILE__, __LINE__, OSIP_ERROR, NULL,
                    "eXosip: Cannot start thread!\n"));
-      return -1;
+	  return -1;
+	}
     }
 #endif
+
   return 0;
 }
 
@@ -634,6 +619,7 @@ eXosip_init (void)
   memset (&eXosip, 0, sizeof (eXosip));
 
   snprintf(eXosip.ipv4_for_gateway, 256, "%s", "217.12.3.11");
+  snprintf(eXosip.ipv6_for_gateway, 256, "%s", "2001:638:500:101:2e0:81ff:fe24:37c6");
 #ifndef MINISIZE
   snprintf(eXosip.event_package, 256, "%s", "dialog");
 #endif
@@ -694,11 +680,11 @@ eXosip_init (void)
   /* open a TCP socket to wake up the application when needed. */
   eXosip.j_socketctl = jpipe ();
   if (eXosip.j_socketctl == NULL)
-    return -1;
+    return -2;
 
   eXosip.j_socketctl_event = jpipe ();
   if (eXosip.j_socketctl_event == NULL)
-    return -1;
+    return -3;
 #endif
 
   /* To be changed in osip! */
@@ -706,6 +692,15 @@ eXosip_init (void)
   osip_fifo_init (eXosip.j_events);
 
   eXosip.use_rport = 1;
+
+  eXosip.keep_alive = 32000;
+
+  eXtl_udp.tl_init();
+  eXtl_tcp.tl_init();
+#ifdef HAVE_OPENSSL_SSL_H
+  eXtl_dtls.tl_init();
+  eXtl_tls.tl_init();
+#endif
   return 0;
 }
 
@@ -901,6 +896,17 @@ eXosip_set_option (eXosip_option opt, const void *value)
                      eXosip.ipv4_for_gateway));
         break;
 #ifndef MINISIZE
+      case EXOSIP_OPT_SET_IPV6_FOR_GATEWAY:
+        tmp = (char *) value;
+        memset (eXosip.ipv6_for_gateway, '\0',
+                sizeof (eXosip.ipv6_for_gateway));
+        if (tmp != NULL && tmp[0] != '\0')
+          strncpy (eXosip.ipv6_for_gateway, tmp, sizeof (eXosip.ipv6_for_gateway));
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                     "eXosip option set: ipv6_for_gateway:%s!\n",
+                     eXosip.ipv6_for_gateway));
+        break;
       case EXOSIP_OPT_EVENT_PACKAGE:
         tmp = (char *) value;
         memset (eXosip.event_package, '\0',
@@ -925,9 +931,6 @@ _eXosip_keep_alive (void)
 {
   static struct timeval mtimer = { 0, 0 };
 
-  eXosip_reg_t *jr;
-  struct eXosip_net *net;
-  char buf[4] = "jaK";
   struct timeval now;
 
   osip_gettimeofday (&now, NULL);
@@ -948,25 +951,7 @@ _eXosip_keep_alive (void)
   osip_gettimeofday (&mtimer, NULL);
   add_gettimeofday (&mtimer, eXosip.keep_alive);
 
-  net = &eXosip.net_interfaces[0];
-  if (net == NULL)
-    {
-      return;
-    }
-
-  for (jr = eXosip.j_reg; jr != NULL; jr = jr->next)
-    {
-      if (jr->len > 0)
-        {
-          if (sendto (net->net_socket, (const void *) buf, 4, 0,
-                      (struct sockaddr *) &(jr->addr), jr->len) > 0)
-            {
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                           "eXosip: Keep Alive sent on UDP!\n"));
-            }
-        }
-    }
+  eXtl_udp.tl_keepalive();
 }
 
 #ifdef OSIP_MT
