@@ -57,6 +57,7 @@
 #include <osip2/osip_dialog.h>
 
 #include <eXosip2/eXosip.h>
+#include "eXtransport.h"
 
 #include "jpipe.h"
 
@@ -79,7 +80,7 @@
 
 #endif
 
-#define EXOSIP_VERSION	"3.0.2"
+#define EXOSIP_VERSION	"3.0.3"
 
 #ifdef __cplusplus
 extern "C"
@@ -235,11 +236,11 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
     char *r_registrar;          /* registrar */
     char *r_contact;            /* list of contacts string */
 
+    char r_line[16];            /* line identifier */
+
     osip_transaction_t *r_last_tr;
     int r_retry;                /* avoid too many unsuccessfull retry */
 
-    char transport[10];         /* transport used for registration */
-
     struct __eXosip_sockaddr addr;
     int len;
 
@@ -354,17 +355,18 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
 #define EXOSIP_MAX_SOCKETS 200
 #endif
 
+#if 0
   /* structure used for keepalive management with connected protocols (TCP or TLS) */
   struct eXosip_socket
   {
     int socket;
-    char remote_ip[50];
+    char remote_ip[65];
     int remote_port;
   };
 
   struct eXosip_net
   {
-    char net_firewall_ip[50];   /* ip address to use for masquerading contacts */
+    char net_firewall_ip[65];   /* ip address to use for masquerading contacts */
     int net_ip_family;          /* AF_INET6 or AF_INET */
     struct sockaddr_storage ai_addr;
     char net_port[20];          /* port for receiving message/connection */
@@ -372,6 +374,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
     int net_protocol;           /* initial socket for receiving message/connection */
     struct eXosip_socket net_socket_tab[EXOSIP_MAX_SOCKETS];
   };
+#endif
 
   char *_eXosip_transport_protocol (osip_message_t * msg);
   int _eXosip_find_protocol (osip_message_t * msg);
@@ -404,7 +407,11 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
 
   struct eXosip_t
   {
+#if 0
     struct eXosip_net net_interfaces[3];
+#endif
+    struct eXtl_protocol *eXtl;
+    char transport[10];
     char *user_agent;
 
     eXosip_call_t *j_calls;     /* my calls        */
@@ -446,6 +453,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
 #endif
     int use_rport;
     char ipv4_for_gateway[256];
+    char ipv6_for_gateway[256];
 #ifndef MINISIZE
     char event_package[256];
 #endif
@@ -529,7 +537,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
   int eXosip_reg_init (eXosip_reg_t ** jr, const char *from,
                        const char *proxy, const char *contact);
   void eXosip_reg_free (eXosip_reg_t * jreg);
-  int generating_register (osip_message_t ** reg, char *transport, char *from,
+  int generating_register (eXosip_reg_t * jreg, osip_message_t ** reg, char *transport, char *from,
                            char *proxy, char *contact, int expires);
 
   int _eXosip_call_transaction_find (int tid, eXosip_call_t ** jc,
@@ -656,6 +664,9 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
 
   int _eXosip_srv_lookup(osip_transaction_t * tr, osip_message_t * sip, struct osip_srv_record *record);
 
+  int _eXosip_handle_incoming_message (char *buf, size_t len, int socket,
+				       char *host, int port);
+
 #ifdef __cplusplus
 }
 #endif
