@@ -114,10 +114,6 @@ cb_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
   int i;
   osip_via_t *via;
 
-  if (eXosip.net_interfaces[0].net_socket == 0
-      && eXosip.net_interfaces[1].net_socket == 0)
-    return -1;
-
 #ifndef MINISIZE
   if(eXosip.dontsend_101 != 0 && sip->status_code == 101)
         return 0;
@@ -200,11 +196,22 @@ cb_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
   i = -1;
   if (osip_strcasecmp (via->protocol, "udp") == 0)
     {
-      i = cb_udp_snd_message (tr, sip, host, port, out_socket);
-  } else
-    {
-      i = cb_tcp_snd_message (tr, sip, host, port, out_socket);
+      i = eXtl_udp.tl_send_message (tr, sip, host, port, out_socket);
     }
+  else if (osip_strcasecmp (via->protocol, "tcp") == 0)
+    {
+      i = eXtl_tcp.tl_send_message (tr, sip, host, port, out_socket);
+    }
+#ifdef HAVE_OPENSSL_SSL_H
+  else if (osip_strcasecmp (via->protocol, "tls") == 0)
+    {
+      i = eXtl_tls.tl_send_message (tr, sip, host, port, out_socket);
+    }
+  else if (osip_strcasecmp (via->protocol, "dtls-udp") == 0)
+    {
+      i = eXtl_dtls.tl_send_message (tr, sip, host, port, out_socket);
+    }
+#endif
   if (i != 0)
     {
       return -1;
@@ -214,6 +221,7 @@ cb_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
 
 }
 
+#if 0
 int
 cb_udp_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
                     int port, int out_socket)
@@ -536,6 +544,7 @@ cb_tcp_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
   return 0;
 
 }
+#endif
 
 static void
 cb_xixt_kill_transaction (int type, osip_transaction_t * tr)
@@ -567,21 +576,51 @@ cb_xixt_kill_transaction (int type, osip_transaction_t * tr)
       eXosip_dialog_t *jd;
       eXosip_subscribe_t *js;
       eXosip_notify_t *jn;
+      eXosip_call_t *jc;
       jinfo_t *jinfo = (jinfo_t *) osip_transaction_get_your_instance (tr);
       
+      if (jinfo == NULL && tr->last_response == NULL)
+	{
+	  eXosip_event_t *je;
+	  je = eXosip_event_init_for_message (EXOSIP_MESSAGE_REQUESTFAILURE, tr);
+	  report_event (je, NULL);
+	  return;
+	}
+
       if (jinfo == NULL)
-	return;
+	{
+	  return;
+	}
+
+      jc = jinfo->jc;
       jd = jinfo->jd;
       jn = jinfo->jn;
       js = jinfo->js;
       
       if (jn == NULL && js == NULL)
-	return;
+	{
+	  eXosip_event_t *je;
+	  if (jc!=NULL)
+	    {
+	      report_call_event (EXOSIP_CALL_MESSAGE_REQUESTFAILURE, jc, jd, tr);
+	      return;
+	    }
+
+	  je = eXosip_event_init_for_message (EXOSIP_MESSAGE_REQUESTFAILURE, tr);
+	  report_event (je, NULL);
+	  return;
+	}
       
       /* no answer to a NOTIFY request! */
       if (MSG_IS_NOTIFY (tr->orig_request) && tr->last_response == NULL)
 	{
 	  /* delete the dialog! */
+	  eXosip_event_t *je;
+	  
+	  je = eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+					     jn, jd, tr);
+	  report_event (je, NULL);
+
 	  REMOVE_ELEMENT (eXosip.j_notifies, jn);
 	  eXosip_notify_free (jn);
 	  return;
@@ -617,6 +656,12 @@ cb_xixt_kill_transaction (int type, osip_transaction_t * tr)
       /* no answer to a SUBSCRIBE request! */
       if (MSG_IS_SUBSCRIBE (tr->orig_request) && tr->last_response == NULL)
 	{
+	  eXosip_event_t *je;
+	  
+	  je = eXosip_event_init_for_subscribe (EXOSIP_SUBSCRIPTION_REQUESTFAILURE,
+						js, jd, tr);
+	  report_event (je, NULL);
+
 	  /* delete the dialog! */
 	  REMOVE_ELEMENT (eXosip.j_subscribes, js);
 	  eXosip_subscribe_free (js);
@@ -903,38 +948,6 @@ __eXosip_new_jinfo (eXosip_call_t * jc, eXosip_dialog_t * jd)
   return ji;
 }
 
-static void
-_eXosip_learn_port_from_via(osip_transaction_t * tr, osip_message_t * sip);
-
-static void
-_eXosip_learn_port_from_via(osip_transaction_t * tr, osip_message_t * sip)
-{
-    /* EXOSIP_OPT_UDP_LEARN_PORT option set */ 
-    if (eXosip.learn_port > 0)
-    {
-        struct eXosip_net *net;
-        osip_via_t *via = NULL;
-        osip_generic_param_t *br;
-
-        net = &eXosip.net_interfaces[0];
-
-        osip_message_get_via (sip, 0, &via);
-        if (via != NULL && via->protocol != NULL
-            && osip_strcasecmp (via->protocol, "udp") == 0)
-        {
-            osip_via_param_get_byname (via, "rport", &br);
-            if (br != NULL && br->gvalue != NULL)
-            {
-                snprintf (net->net_port, 20, "%s", br->gvalue);
-                OSIP_TRACE (osip_trace
-                            (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                            "_eXosip_learn_port_from_via (id=%i) SIP port modified from rport in SIP answer\r\n",
-                            tr->transactionid));
-            }
-        }
-    }
-    return;
-}
 
 static void
 cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
@@ -951,7 +964,7 @@ cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv1xx (id=%i)\r\n",
                tr->transactionid));
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
   if (jinfo == NULL)
     return;
@@ -1329,7 +1342,7 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv2xx (id=%i)\r\n",
                tr->transactionid));
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
 #ifndef MINISIZE
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
@@ -1361,6 +1374,19 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       _eXosip_reg_find (&jreg, tr);
       if (jreg != NULL)
         {
+	  /* update registration interval */
+	  osip_header_t *exp;
+	  
+	  osip_message_header_get_byname (sip, "expires", 0, &exp);
+	  if (exp!=NULL && exp->hvalue!=NULL)
+	    {
+	      int val = atoi(exp->hvalue);
+	      if (val>0)
+		{
+		  jreg->r_reg_period=val+60;
+		}
+	    }
+
           je = eXosip_event_init_for_reg (EXOSIP_REGISTRATION_SUCCESS, jreg, tr);
           report_event (je, sip);
           jreg->r_retry = 0;    /* reset value */
@@ -1502,7 +1528,7 @@ cb_rcv3xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv3xx (id=%i)\r\n",
                tr->transactionid));
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
@@ -1598,7 +1624,7 @@ cb_rcv4xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv4xx (id=%i)\r\n",
                tr->transactionid));
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
@@ -1707,7 +1733,7 @@ cb_rcv5xx (int type, osip_transaction_t * tr, osip_message_t * sip)
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv5xx (id=%i)\r\n",
                tr->transactionid));
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
@@ -1798,7 +1824,7 @@ cb_rcv6xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   eXosip_notify_t *jn;
   jinfo_t *jinfo = (jinfo_t *) osip_transaction_get_your_instance (tr);
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
   OSIP_TRACE (osip_trace
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv6xx (id=%i)\r\n",
@@ -1902,7 +1928,7 @@ cb_rcv3456xx (int type, osip_transaction_t * tr, osip_message_t * sip,
               (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv3456xx (id=%i)\r\n",
                tr->transactionid));
 
-  _eXosip_learn_port_from_via(tr, sip);
+  udp_tl_learn_port_from_via(sip);
 
   if (MSG_IS_RESPONSE_FOR (sip, "REGISTER"))
     {
@@ -2012,24 +2038,24 @@ cb_snd123456xx (int type, osip_transaction_t * tr, osip_message_t * sip)
   jc = jinfo->jc;
   if (jd == NULL)
     return;
-  if (OSIP_IST_STATUS_1XX_SENT || OSIP_NIST_STATUS_1XX_SENT)
+  if (type == OSIP_IST_STATUS_1XX_SENT || type == OSIP_NIST_STATUS_1XX_SENT)
     {
       jd->d_STATE = JD_TRYING;
       return;
     }
-  if (OSIP_IST_STATUS_2XX_SENT || OSIP_NIST_STATUS_2XX_SENT)
+  if (type == OSIP_IST_STATUS_2XX_SENT || type == OSIP_NIST_STATUS_2XX_SENT)
     {
       jd->d_STATE = JD_ESTABLISHED;
       return;
     }
   
-  if (OSIP_IST_STATUS_3XX_SENT || OSIP_NIST_STATUS_3XX_SENT)
+  if (type == OSIP_IST_STATUS_3XX_SENT || type == OSIP_NIST_STATUS_3XX_SENT)
     jd->d_STATE = JD_REDIRECTED;
-  else if (OSIP_IST_STATUS_4XX_SENT || OSIP_NIST_STATUS_4XX_SENT)
+  else if (type == OSIP_IST_STATUS_4XX_SENT || type == OSIP_NIST_STATUS_4XX_SENT)
     jd->d_STATE = JD_CLIENTERROR;
-  else if (OSIP_IST_STATUS_5XX_SENT || OSIP_NIST_STATUS_5XX_SENT)
+  else if (type == OSIP_IST_STATUS_5XX_SENT || type == OSIP_NIST_STATUS_5XX_SENT)
     jd->d_STATE = JD_SERVERERROR;
-  else if (OSIP_IST_STATUS_6XX_SENT || OSIP_NIST_STATUS_6XX_SENT)
+  else if (type == OSIP_IST_STATUS_6XX_SENT || type == OSIP_NIST_STATUS_6XX_SENT)
     jd->d_STATE = JD_GLOBALFAILURE;
   
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE")
